@@ -207,7 +207,8 @@ sub set_handler {
 sub set_connection_handler {
     my($self, $module, $args) = @_;
     my $port = $self->new_vhost($module);
-    $self->postamble(Listen => $port);
+    my $vars = $self->{vars};
+    $self->postamble(Listen => $vars->{servername} . ':' . $port);
 }
 
 my %add_hook_config = (
@@ -240,7 +241,7 @@ sub new_vhost {
     my($port, $servername, $vhost);
 
     unless ($namebased and exists $self->{vhosts}->{$module}) {
-        $port       = $self->server->select_port;
+        $port       = $self->server->select_next_port;
         $vhost      = $self->{vhosts}->{$module} = {};
 
         $vhost->{port}       = $port;
